@@ -7,6 +7,7 @@ import os.path
 from abc import abstractmethod
 from os import PathLike
 from typing import Dict, List, Optional, Set, Union
+from xml.etree.ElementTree import Element as XMLElement  # nosec B405
 
 from quark.core.struct.bytecodeobject import BytecodeObject
 from quark.core.struct.methodobject import MethodObject
@@ -69,6 +70,16 @@ class BaseApkinfo:
         """
         pass
 
+    @property
+    @abstractmethod
+    def activities(self) -> List[XMLElement]:
+        """
+        Return all activity from given APK.
+
+        :return: a list of all activities
+        """
+        pass
+
     @property
     @abstractmethod
     def android_apis(self) -> Set[MethodObject]:
