@@ -118,7 +118,7 @@ class RadioContrast:
                     "score": 1,
                     "label": [],
                 }
-                comb = RuleObject("test", json_data=generated_rule)
+                comb = RuleObject("test", jsonData=generated_rule)
 
                 try:
                     self.quark.run(comb)
